/*********************************************************************/
/*                                                                   */
/*   Program Name :  data_recov                                      */
/*                                                                   */
/*      Version:  1.23  2008-02-19                                   */
/*                                                                   */
/*                                                                   */
/*      Header data edit or recovering for K5/VSSP and VSSP32 data   */
/*                                                                   */
/*                                                                   */
/*      Copyright (c) 2006-2008 T.Kondo/NICT All Right Reserved      */
/*                                                                   */
/*********************************************************************/
/*
      usage

         data_recov [stype] K5file [sfreq adbit numch start_time [HDpat1 HDpat2]]
                    [options]
       
              where  
                     stype     --- '32' for VSSP32, '0' for VSSP mode
                                    (default or omitted case is VSSP)
                     K5file   ----- K5 data file name to edit
                                    Output file name shall be K5file+'.edit'
                                    unless '-out utfile' option is set 

                     *following parameters are for special header search mode
                     sfreq    ----- Sampling frequency (MHz)
                                    0.04,0.1,0.2,0.5,1,2,4,8,16,32,64
                     adbit    ----- A/D resolution (1,2,4,8)
                     numch    ----- # of channels (1,4)
                     start_time --- 1st time of data
	                                either "HH:MM:SS" or "seconds in day"
                     HDpat1  ------ corrupted header pattern 1 in 8 digit Hexs or '?'
                                    where '?' is a wild card for one Hex.
                                    (ex. 'FFFFFF??') (default is 'FFFF????')
                     HDpat2  ------ corrupted header pattern 3 in 8 digit Hexs or '?'
                                    where '?' is a wild card for one Hex.
                                    (ex. '8B????FF') 
                                    (default is '8B??????' for VSSP,
                                                '8C??????' for VSSP32)
        
              options (any order)
                  -recover --- force recover mode (non-interactive operation)
                  -check  ---- force check mode (non-interactive operation)
                  -pat1 HDpat1  --- set serach header pattern 1
                  -pat2 HDpat2  --- set serach header pattern 2
                  -vssp32 ---- set sampler type to VSSP32 (default is VSSP)
                  -out ofile --- set output data file name after corrected
                  -in k5file --- set K5 file name to check or edit
                  -offset nbytes --- set offset bytes compulsory



        ver 0.00 2006-07-23  start development 
        ver 1.00 2006-07-24  release 
        ver 1.10 2006-09-20  Separate usuall header mode and corruppted header search mode 
        ver 1.11 2006-09-21  Add user defined serach header pattern
        ver 1.12 2006-02-22  update due to libvssp32.h update
        ver 1.13 2006-04-10  use '\?' instead of '?' in printf function to avoid unnecessary warning
        ver 1.20 2007-10-19  support VSSP32
        ver 1.21 2007-10-20  add options in command line to support full non-interactive operation 
        ver 1.22 2007-10-23  more robust against the data those 1st header is corupted
                             add '-offset nbytes' in command option
        Ver 1.23 2008-02-19  option '--help' and '--ver[sion]' were added


F:/datacheck/t2044_kb1_144-0453_2006144045324.k5
F:/datacheck/t2044_kb1_144-1527_2006144152738.k5
F:/datacheck/t2044_kb1_144-1705a_2006144170520.k5
D:\IPVLBI\data\temp\O202151500a.dat
D:\IPVLBI\data\GSIcheck\rdv58_ts2_242-1800a_2006242180000.k5 16 1 4 17:59:31
32 D:\IPVLBI\data\errordata\errorGSIsample.k5 16 1 4 61005
*/
#define PROGRAM_NAME "data_recov"
#define PROGRAM_VERSION "Ver. 2008-02-19"

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libvssp32.h"

typedef struct data_recov_runp    /* run parameter structure */
{            
    char fname[80];
	char oname[100];
    char hdpat1[20];
    char hdpat2[20];
    int byteoffset;
    int mode;
    int pmode; // processing mode
    int interactive_mode;  // 0: non-interactive  1: interactive
    int edit_mode;    // 1: edit mode  0: no edit mode
    int check_mode;    // 1: check mode  0: no check mode
	int hh,mm,ss;
	double fsmhz;
    int compulsory_set_flag;  // sampling parameter and 1st time set compulsory 
                              // from operator
    int offset_compulsory_set_flag;
} data_recov_runp_t;

void monit_vssp_hd(vssp_head_t *vph)
{
    /* VSSP headder monitor */

    int n;
	u_int32_t *head32; // **K5**  to contain full header including 
    vsspaux_t vsspaux;

    head32=(u_int32_t *)vph->bhead;
    vsspaux.size=vph->aux_size;
    for(n=0; n<vsspaux.size; n++) vsspaux.bdat[n]=vph->bhead[12+n];
    printf("format aux_fmt,filter %d %d %d\n",vph->format,vph->aux_fmt,vph->filter);
    printf("before sid2 = %s\n",vph->aux_sid2);
    printf("before station = %s\n",vph->aux_station);
    printf("before host    = %s\n",vph->aux_hostname);
    vssp_aux_parse(&vsspaux);
    printf("aux_fmt = %d   filter = %d\n",vsspaux.fmt,vsspaux.filter);
    printf("sid2 = %s\n",vsspaux.sid2);
    printf("station = %s\n",vsspaux.station8);
    printf("host    = %s\n",vsspaux.hostname8);
}




void vssp_hd_make_ini(vssp_head_t *vph)
{
    /* VSSP headder common information set up */
    int n,fid,aid;
    long int w1;
    unsigned char b,c;
	u_int32_t *head32; // **K5**  to contain full header including 
    head32=(u_int32_t *)vph->bhead;

    //double fsampl;
    /* sync pattern 1 set */
    for (n=0; n<4; n++){
       vph->bhead[n]=0xFF;
    }
    /* seconds set */
    w1=vph->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    vph->bhead[4]=(unsigned char)(w1 & 0xFF);
    vph->bhead[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=(unsigned char)((w1 & 0x10000) >> 16);
    /* ch flag set */
    if( vph->numch == 4){
        b=b | 0x2;
    }
    /* sampling freq index set */
    fid=0;
    //if( ipb->sfreq ==    40) fid=0;
    if( vph->sfreq ==   100) fid=1;
    if( vph->sfreq ==   200) fid=2;
    if( vph->sfreq ==   500) fid=3;
    if( vph->sfreq ==  1000) fid=4;
    if( vph->sfreq ==  2000) fid=5;
    if( vph->sfreq ==  4000) fid=6;
    if( vph->sfreq ==  8000) fid=7;
    if( vph->sfreq == 16000) fid=8;
    if( vph->sfreq == 32000) fid=9;
    if( vph->sfreq == 64000) fid=10;
    if( vph->sfreq == 128000) fid=11; 
    if( vph->sfreq == 256000) fid=12;
    if( vph->sfreq == 512000) fid=13;
    if( vph->sfreq == 1024000) fid=14;
    if( vph->sfreq == 2048000) fid=15;
    c=(unsigned char)(fid << 2);
    b=b | c;
    /* A/D bit length set */
    aid=0;
    //if( ipb->adbit == 1 ) aid=0;
    if( vph->adbit == 2 ) aid=1;
    if( vph->adbit == 4 ) aid=2;
    if( vph->adbit == 8 ) aid=3;
    c=(unsigned char)(aid << 6);
    b=b | c;
    vph->bhead[6]=b;
    /* sync pattern 2 set */
    //vph->bhead[7]=0x8B;
    vph->bhead[7]=vph->psync2;

    /* extended header set for VSSP32 */
    if(vph->format == 2){   // VSSP32
       head32[2]= ((vph->year-2000) & 0x3F) << 9 ;  // year
       head32[2]= (head32[2] & 0xFFFFFE00) | (vph->totalday & 0x1FF) ;  // total day
       head32[2]= (head32[2] & 0x0FFFFFFF) | ((vph->v_major & 0xF) << 28) ;  // major version#
       head32[2]= (head32[2] & 0xF0FFFFFF) | ((vph->v_minor & 0xF) << 24) ;  // minor version#
       head32[2]= (head32[2] & 0xFF00FFFF) | ((vph->aux_size & 0xFF) << 16) ;  // aux size
       switch(vph->aux_fmt){
       case 0:
           for(n=0; n<5; n++) head32[3+n]=0;
           break;
       case 1:
           //head32[3]=((vph->aux_fmt & 0xFF) << 8) | (vph->filter & 0xFF) ;
           head32[3]=(vph->aux_fmt & 0xFF) | ((vph->filter & 0xFF) << 8) ;
           strncpy(vph->bhead+14,vph->aux_sid2,2);
           strncpy(vph->bhead+16,vph->aux_station,8);
           strncpy(vph->bhead+24,vph->aux_hostname,8);
           //for(n=0;n<2;n++) vph->bhead[14+n]=vph->aux_sid2[n];
           //for(n=0;n<8;n++) vph->bhead[16+n]=vph->aux_station[n];
           //for(n=0;n<8;n++) vph->bhead[24+n]=vph->aux_hostname[n];
           break;
       case 2:
           head32[3]=((vph->aux_fmt & 0xFF) << 8) | (vph->filter & 0xFF) ;
           for(n=0; n<10; n++) vph->bhead[14+n]=0x55;
           strncpy(vph->bhead+24,vph->aux_hostname,8);
           break;
       case 85:
           head32[3]=((vph->aux_fmt & 0xFF) << 8) | (vph->filter & 0xFF) ;
           for(n=0; n<18; n++) vph->bhead[14+n]=0x55;
           break;
       case 170:
           head32[3]=((vph->aux_fmt & 0xFF) << 8) | (vph->filter & 0xFF) ;
           for(n=0; n<18; n++) vph->bhead[14+n]=0xAA;
           break;
       default:
           for(n=0; n<5; n++) head32[3+n]=0;
       }
    }
    /* internal data initialize */
//    fsampl=1000.0*(double)ipb->sfreq; // sampling frequency (Hz)
//
//    ipb->nbsec=(((long)fsampl)*ipb->adbit*ipb->numch)/8;  // # of bytes in 1 sec data except header
//    ipb->nbsec+=8;
//    ipb->bpos=0;
}

void vssp_hd_make(vssp_head_t *vph)
{
    /* VSSP data headder second data up date */
    long int w1;
    unsigned char b,c;
    /* seconds set */
    w1=vph->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    vph->bhead[4]=(unsigned char)(w1 & 0xFF);
    vph->bhead[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=vph->bhead[6];
    c=(unsigned char)((w1 & 0x10000) >> 16);
    b=(b & 0xFE) | (c & 0x1);
    vph->bhead[6]=b;
}

int cont_file_open(FILE *fi, char *fname, int *f_c, char *k5name, int64_t *size)
{     
       /* continuous file open */
       char fname2[80];
       char cbuf[80];

       *f_c++;
       // continuous file name creation     XXXX.datN   where N=1,2,3......
       strcpy(fname2,fname);
       sprintf(cbuf,"%d\0",*f_c);
       strcat(fname2,cbuf);
       strcpy(k5name,fname2);
       *size=get_file_size(k5name);
       // file open 
       if( (fi  = fopen( k5name, "rb" )) == NULL ) {
           //printf("File Open Error! \n");
           return -1;
       }
       printf("\n continuous file (%s (%d bytes)) opened!\n",k5name,*size);  
       printf("                                    "); 
       return 0;
}

void patternconversion(char* hpat8, u_int32_t *pattern, u_int32_t *mask)
{
    /*
         Convert 8 digit HexAscii expression (eg. 8B????FF )
         to pattern ( 0x8B0000FF ) and mask ( FF0000FF )
    */
    int n,len;
    int t1,t2;
    u_int32_t w1,w2;
    len=strlen(hpat8);
    if(len > 8) len=8;
    w1=0; w2=0;
    for(n=0; n<len; n++){
        t1=0;
        t2=0;
        if(hpat8[n] >= 0x30 && hpat8[n] <= 0x39){  //  ASCII '0',....,'9' 
            t1=(int)(hpat8[n]-0x30);
            t2=0xF;
        } 
        if(hpat8[n] >= 0x41 && hpat8[n] <= 0x46){  //  ASCII 'A',....,'F' 
            t1=(int)(hpat8[n]-0x41)+10;
            t2=0xF;
        }
        if(hpat8[n] >= 0x61 && hpat8[n] <= 0x66){  //  ASCII 'a',....,'f' 
            t1=(int)(hpat8[n]-0x61)+10;
            t2=0xF;
        }
        //if(hpat8[n] == 0x3F){}  //  ASCII '?' 
        //if(hpat8[n] == 0x58){}  //  ASCII 'X' 
        //if(hpat8[n] == 0x78){}  //  ASCII 'x' 
        //if(hpat8[n] == 0x2A){}  //  ASCII '*' 
        w1=(w1<<4)|t1;
        w2=(w2<<4)|t2;
    }
    *pattern=w1;
    *mask=w2;
}




int find_offset(char* fname, vssp_head_t *vph, int mode, 
                char* hpat1, char* hpat2) 
{
       /* find out possible byte offset */
       int i;
       int64_t tword;
       long int numb,npos, nkai;
       long int bitsec;
       int headerbits;
       int total_frame, bad_frame;
       char k5name[80],cbuf[10];
       double fsample;
       int bytesec,wordsec;
	   u_int32_t ubuf1,ubuf2;
       int wordread;
	   u_int32_t pattern1,pattern2,mask1,mask2;
	   int sa,ipos;
	   int byteoffset,iok;
	   int64_t pos,posold;   // current pointer
	   int64_t size;  // opened file size in bytes **K5**
       FILE *fi;
       //fpos_t size;

       total_frame=0;
       bad_frame=0;
       numb=0;
       npos=0;
       nkai=0;
	   byteoffset=-1;
	   iok=0;

       headerbits=64;  // for K5/VSSP
       if(mode==32) headerbits=96+20*8;  // for K5/VSSP32 usually 256

       /* calculation of datasize in 1sec */

       fsample=1000.0*vph->sfreq;   // sampling frequency (Hz)
       //K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)fsample*vph->adbit*vph->numch;  // # of bits in 1 sec data except header
       bytesec=bitsec/8;                  //# of bytes in 1 sec data
       wordsec=bitsec/32;                 //# of bytes in 1 sec data

       strcpy(k5name,fname);

       /* K5/VSSP file open */

       size=get_file_size(k5name);
       if( (fi  = fopen( k5name, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",k5name);
           goto exitend2;
       }
       printf("FILE : %s  (%u bytes)\n",k5name,size);

       //K5.twordread=0;  // reset word read counter
       pos=0;        // reset current pointer position
	   posold=0;
       tword=0;
       

       /* initial pattern serach (this may be good for k51a only?) */
	   /*
             Broken headder usually has a pattern as follows
			 
			   FFFF???? 8B????FF

                (this may be good for k51a only?)

	   */
	   //pattern1=0xFFFF0000;  // pattern for k51a at Kashima
	   //pattern2=0x8B0000FF;  // pattern for k51a at Kashima
	   //mask1 = 0xFFFF0000;  // mask for k51a at Kashima
	   //mask2 = 0xFF0000FF;  // mask for k51a at Kashima

       patternconversion(hpat1, &pattern1, &mask1);
       patternconversion(hpat2, &pattern2, &mask2);

	   printf("Pattern %08X %08X \n",pattern1,pattern2);
	   printf("Mask    %08X %08X searching....\n",mask1,mask2);
	   nkai=0;
       wordread = fread( &ubuf1, sizeof( u_int32_t ), 1, fi );
       while(1){
          wordread = fread( &ubuf2, sizeof( u_int32_t ), 1, fi );
          if( ferror( fi ) || wordread==0 ){ // File EOF or read error
             goto exitend;
          }
          if(((ubuf1 & mask1)== (pattern1 & mask1))
			  &&((ubuf2 & mask2)== (pattern2 & mask2))){
			  iok=1;
		      sa=(int)(pos-posold);
		      cbuf[0]='\0';
              if(sa == bytesec+headerbits/8) strcpy(cbuf,"GOOD");
			  ipos=(int)pos;
              printf("%5d Pattern found %08X %08X at pos=%12d  sa=%10d %s\n",
				  nkai,ubuf1,ubuf2,ipos,sa,cbuf);
              //printf("Posold=%lld\n",posold );
              //printf("sa=%d\n",sa );
			  if(nkai == 0) byteoffset=ipos;
		      posold=pos;
			  nkai++;
			  if(nkai==10) goto exitend;
          }
          pos+=4;
		  ubuf1=ubuf2;
       }
exitend:
       i=fclose( fi );
	   if(iok==1){
		   printf(" Found byte offset is %d\n",byteoffset);
	   } else {
		   printf(" Sorry! No pattern found! I gave up!\n");
	   }
exitend2:
	   return byteoffset;

}


void checkK5file(data_recov_runp_t *runp, vssp_head_t *vph) 
{
       int i;
       int64_t tword;
       long int bitsec;
       int headerbits;
       int f_c;
       int total_frame;
       int mode,byteoffset;
       char k5name[80];
       double fsample;
       int bytesec,wordsec;
       u_int32_t ubuf;
       int wordread;
	   int64_t pos,posold;   // current pointer
	   int64_t size;  // opened file size in bytes **K5**
       FILE *fi;
       //fpos_t size;

       mode=runp->mode;
       byteoffset=runp->byteoffset;

       total_frame=0;
       f_c=0;

       headerbits=64;  // for K5/VSSP
       if(mode==32) headerbits=96+20*8;  // for K5/VSSP32 usually 256

       /* calculation of datasize in 1sec */

       fsample=1000.0*vph->sfreq;   // sampling frequency (Hz)
       //K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)fsample*vph->adbit*vph->numch;  // # of bits in 1 sec data except header
       bytesec=bitsec/8;                  //# of bytes in 1 sec data
       wordsec=bitsec/32;                 //# of bytes in 1 sec data


       strcpy(k5name,runp->fname);

       /* K5/VSSP file open */

       size=get_file_size(k5name);
       if( (fi  = fopen( k5name, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",k5name);
           goto exitend2;
       }
       //printf("FILE : %s  (%lld bytes)\n",k5name,size);
       printf("FILE : %s  (%u bytes)\n",k5name,size);

       //K5.twordread=0;  // reset word read counter
       pos=0;        // reset current pointer position
	   posold=0;
       tword=0;
       
       /* move pointer to initial position */
       if(byteoffset > 0){
          fseek(fi,byteoffset,SEEK_CUR); // move pointer by K5->bytesec
          if( ferror( fi ) ){
              printf("set initial position error!\n");
              exit(0);
          }
       }
       pos+=byteoffset;   // current pointer position update

       printf("\n");
       printf("FRAM#         HEADER %dbytes data\n",headerbits/8);
       printf("-----     --------------------------------\n");

loop1:

       /* supposed header data read */

       /* 64bit or 256bit data read by 32bit step */

       printf(" %04d  ",total_frame);
       for(i=0; i<headerbits/32; i++){
          wordread = fread( &ubuf, sizeof( u_int32_t ), 1, fi );
          if( ferror( fi ) || wordread==0 ){ // File EOF or read error
             i=fclose( fi ) ;
             // next file candidate open
             i=cont_file_open(fi,runp->fname,&f_c,k5name,&size);
             if(i < 0) goto exitend2;
             pos=0;
             wordread = fread( &ubuf, sizeof( u_int32_t ), 1, fi );
             if( ferror( fi ) || wordread==0 ) goto exitend;
          }
          printf(" %08X",ubuf); 
          pos+=4;  // 4 bytes count up
       }
       printf("\n");


       /* move 1sec data */

       pos+=bytesec;  // pointer position to be set
       if(pos < size){  // not yet reached to EOF
          fseek(fi,bytesec,SEEK_CUR); // move pointer by bytesec
          if( ferror( fi ) ){
             i=fclose( fi ) ;
             goto exitend2;
          }
       } else {  // exceed EOF so pointer set to EOF
          //fseek(fi,(int)(size - pos),SEEK_CUR); // 
          //if( ferror( fi ) ){
          //   i=fclose( fi ) ;
          //   goto exitend2;
          //}
          pos=pos-size;  // position to be set in the next file
          i=fclose( fi ) ;
          // next file candidate open
          i=cont_file_open(fi,runp->fname,&f_c,k5name,&size);
          if(i < 0) goto exitend2;
          // point next header
          fseek(fi,(int)pos,SEEK_CUR); // 
          if( ferror( fi ) ){
             i=fclose( fi ) ;
             goto exitend2;
          }
       }

       total_frame++;

       goto loop1;
   
 exitend:
       i=fclose( fi ) ;
 exitend2:;
       printf("\n");
}



void sec2hhmmss(int seconds, int *hh, int *mm, int *ss)
{
	//  seconds => HH:MM:SS
	int hh1,mm1,ss1;
    hh1=seconds/3600;
	mm1= (seconds % 3600)/60;
	ss1= seconds % 60;
    *hh=hh1; *mm=mm1; *ss=ss1;
}


void editK5file(data_recov_runp_t *runp, vssp_head_t *vph) 
{
       //int usize=20000;  // ubuf size
       int usize;  // ubuf size
       int i,n,k;
       int64_t tword;
       long int bitsec;
       int headerbits;
       int printbytes;
       int f_c;
       int total_frame, bad_frame;
       int mode,byteoffset;
       int iflag,hh,mm,ss;
       char k5name[80];
       double fsample;
       int bytesec,wordsec,imax;
       u_int32_t *ubuf;
	   u_int32_t *head;
       u_int32_t oldhead[20];
       int wordread,iwrite;
	   int64_t pos,posold;   // current pointer
	   int64_t size;  // opened file size in bytes **K5**
       FILE *fi,*fo;

	   head=(u_int32_t *)vph->bhead;

       mode=runp->mode;
       byteoffset=runp->byteoffset;
       total_frame=0;
       bad_frame=0;

       f_c=0;

       headerbits=64;  // for K5/VSSP
       if(mode==32) headerbits=96+20*8;  // for K5/VSSP32 usually 256
       printbytes=8;  // bytes printed to display in a headder

       /* calculation of datasize in 1sec */

       fsample=1000.0*vph->sfreq;   // sampling frequency (Hz)
       //K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)fsample*vph->adbit*vph->numch;  // # of bits in 1 sec data except header
       bytesec=bitsec/8;                  //# of bytes in 1 sec data
       wordsec=bitsec/32;                 //# of bytes in 1 sec data

       // buffer size fix so as wordsec/usize to become integer number 
       usize=1250;
       if(vph->sfreq >= 100) usize=3125;
       if(vph->sfreq >= 1000) usize=31250;
       imax=wordsec/usize;

       ubuf=(u_int32_t *)malloc(usize*sizeof(u_int32_t));

       strcpy(k5name,runp->fname);

       /* K5/VSSP file open */

       size=get_file_size(k5name);
       if( (fi  = fopen( k5name, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",k5name);
           goto exitend2;
       }
       //printf("FILE : %s  (%lld bytes)\n",k5name,size);
       printf("FILE : %s  (%u bytes)\n",k5name,size);

	   /* Output file open */
       if( (fo  = fopen( runp->oname, "wb" )) == NULL ) {
           printf("File %s Open Error! \n",runp->oname);
           goto exitend2;
       }
	   /* Header initialize */
       vssp_hd_make_ini(vph);

       //monit_vssp_hd(vph);  // debug monitor

//       printf("AUX format filter %d %d\n",vph->bhead[13],vph->bhead[12]);
//       printf("sid2= %c%c\n",vph->bhead[14],vph->bhead[15]);
//       printf("station= %c%c%c%c%c%c%c%c\n",vph->bhead[16],vph->bhead[17],
//              vph->bhead[18],vph->bhead[19],vph->bhead[20],vph->bhead[21],
//              vph->bhead[22],vph->bhead[23]);
//       printf("Host= %c%c%c%c%c%c%c%c\n",vph->bhead[24],vph->bhead[25],
//              vph->bhead[26],vph->bhead[27],vph->bhead[28],vph->bhead[29],
//              vph->bhead[30],vph->bhead[31]);

       //K5.twordread=0;  // reset word read counter
       pos=0;        // reset current pointer position
	   posold=0;
       tword=0;
       
       /* move pointer to initial position */
       if(byteoffset > 0){
          fseek(fi,byteoffset,SEEK_CUR); // move pointer by K5->bytesec
          if( ferror( fi ) ){
              printf("set initial position error!\n");
              exit(0);
          }
       }
       pos+=byteoffset;   // current pointer position update

       printf("\n");
       printf("Frm#       OLD HEADER              NEW HEADER     (HH:MM:SS)\n");
       printf("----   -----------------       -----------------  ----------\n");


loop1:

       /* supposed header data read */

       /* 64bit or 256bit data read by 32bit step */

       printf("%04d  ",total_frame);
       for(i=0; i<headerbits/32; i++){
       //for(i=0; i<printbytes/4; i++){
          wordread = fread( ubuf, sizeof( u_int32_t ), 1, fi );
          if( ferror( fi ) || wordread==0 ){ // File EOF or read error
             i=fclose( fi ) ;
             // next file candidate open
             i=cont_file_open(fi,runp->fname,&f_c,k5name,&size);
             if(i < 0) goto exitend2;
             pos=0;
             wordread = fread( ubuf, sizeof( u_int32_t ), 1, fi );
             if( ferror( fi ) || wordread==0 ) goto exitend;
          }
          if(i < printbytes/4) printf(" %08X",ubuf[0]); 
          oldhead[i]=ubuf[0];
          pos+=4;  // 4 bytes count up
       }

       vssp_hd_make(vph);
	   // header write 
       iwrite=fwrite(vph->bhead,sizeof(char),headerbits/8,fo);
	   printf(" ===> ");
	   //head=(u_int32_t *)vph->bhead;  // move to the begininng point of this function
       iflag=0;
       for(i=0; i<headerbits/32; i++){
          if(i < printbytes/4) printf(" %08X",head[i]);
          if(oldhead[i]!=head[i]) iflag=1;
	   }

       //additional info display
       sec2hhmmss(vph->seconds, &hh, &mm, &ss);
       printf("  (%02d:%02d:%02d)",hh,mm,ss);
       if(iflag == 1){
           printf("  Header corrected");
           bad_frame++;
       } else {
           printf("  OK");
       }
       
       printf("\n");


	   vph->seconds++;   // seconds update
       total_frame++;

       /* copy 1sec data */

	   for(n=0; n<imax; n++){
          wordread = fread( ubuf, sizeof( u_int32_t ), usize, fi );
          if( ferror( fi ) || wordread==0 ){ // File EOF or read error
			 if(wordread > 0) 
             i=fclose( fi ) ;
             // next file candidate open
             i=cont_file_open(fi,runp->fname,&f_c,k5name,&size);
             if(i < 0) goto exitend2;
			 // rest of data read
             k = fread( ubuf, sizeof( u_int32_t ), usize-wordread, fi );
             iwrite=fwrite(ubuf,sizeof(u_int32_t),k,fo);
		  } else {
             iwrite=fwrite(ubuf,sizeof(u_int32_t),wordread,fo);
		  }
	   }


       goto loop1;
   
 exitend:
       i=fclose( fi ) ;
 exitend2:;
       i=fclose( fo ) ;
       free(ubuf);
       printf("\n");
       /* summary display */
       printf("  Total Frame = %d   Corrected Frame = %d\n",total_frame,bad_frame);
       printf("  Created File = %s\n",runp->oname);
       printf("\n");

}

int hhmmss2sec(char *cbuf, int *hh, int *mm, int *ss)
{
	//  HH:MM:SS => seconds
	int n,len;
	int hh1,mm1,ss1,seconds;
	int p1,p2, n1st;
	char buf[10];
	len=strlen(cbuf);
    // check ':' position
	p1=0; p2=0;
	n1st=0;
    //printf("len %d\n",len);
	for(n=0; n<len; n++){
		if(cbuf[n]==':'){
			if(n1st==0){
				p1=n;
				n1st=1;
			} else {
				p2=n;
				n1st=2;
			}
		}
	}
	//printf("p1,p2 %d %d\n",p1,p2);
	if(n1st==0){ // no HH:MM:SS
       seconds=atoi(cbuf);
	   hh1=seconds/3600;
	   mm1= (seconds % 3600)/60;
	   ss1= seconds % 60;
       *hh=hh1; *mm=mm1; *ss=ss1;
	   return seconds;
	}

	strncpy(buf,cbuf,p1);
	buf[p1]='\0';
	hh1=atoi(buf);
	strncpy(buf,cbuf+p1+1,p2-p1-1);
	buf[p2-p1]='\0';
	mm1=atoi(buf);
	strncpy(buf,cbuf+p2+1,len-p2-1);
	buf[len-p2]='\0';
	ss1=atoi(buf);
    seconds=hh1*3600+mm1*60+ss1;
	*hh=hh1; *mm=mm1; *ss=ss1;
	//printf("HH MM SS %d %d %d\n",hh,mm,ss); // debug
	return seconds;
}



int chkK5header(data_recov_runp_t *runp, vssp_head_t *vph) 
{
       int usampl=200000;
       int i,n;
	   int kbytes;
       int64_t tword;
       long int numb,npos, tflag, nkai;
       long int bitsec;
       int headerbits;
       int f_c;
       int total_frame, bad_frame;
       int ierr;
       char fmt[10];
       double dbytes;
       K5VSSP_file_t K5;
       vsspaux_t aux;
       //fpos_t size;

       k5vssp_t_ini(&K5);
       total_frame=0;
       bad_frame=0;
       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;

       ierr=0;

       if(runp->offset_compulsory_set_flag==0)
            runp->byteoffset=0;

       headerbits=64;  // for K5/VSSP

       strcpy(K5.fname,runp->fname);



       /* K5/VSSP file open and 1st header read 
               to get sampling information etc */

       K5.size=get_file_size(K5.fname);
       if( (K5.stream  = fopen( runp->fname, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",K5.fname);
           ierr=-1;
           goto exitend2;
       }
       //printf("FILE : %s  (%lld bytes)\n",K5.fname,K5.size);
       printf("FILE : %s  (%u bytes)\n",K5.fname,K5.size);


       tword=0;

       //if(checkheader(&K5) < 0){
       if((kbytes=CheckK5header_with_Read(&K5)) < 0){
            printf("File read error! \n");
            //fgetpos(K5.stream,&K5.pos); 
            i=fclose( K5.stream ) ;
            ierr=-1;
            goto exitend2;
       }

       if(K5.format==2) headerbits=96+K5.auxbyte*8;  // usually 256
       //printf("headerbits= %d\n",headerbits);

       tword+=K5.twordread;
       K5.twordread=0;

       K5.fsample=1000.0*K5.sfreq;   // sampling frequency (Hz)
       K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)K5.fsample*K5.adbit*K5.numch;  // # of bits in 1 sec data except header
       K5.bytesec=bitsec/8;                  //# of bytes in 1 sec data
       K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second
       K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
       K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl

       K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

       //K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer

       if(kbytes >0){
           printf("First 64bit is not a header block!\n");
       }
       //*byteoffset=kword*4;

       if(runp->offset_compulsory_set_flag==0)
          runp->byteoffset=kbytes;
       if(runp->compulsory_set_flag==0){
          vph->sfreq=K5.sfreq;
          vph->adbit=K5.adbit;
          vph->numch=K5.numch;
          vph->seconds=K5.seconds;
       }
       vph->format=K5.format;




       strcpy(fmt,"VSSP");
       if(K5.format==2) strcpy(fmt,"VS32");

       if(K5.format==2){
           printf("\n This is K5/VSSP32 format data.\n");
           printf("\n  1st Extended Header Info is as follows.\n");
           printf("    Year = %d    Total Day = %d\n", K5.year,K5.totalday);
           printf("    Version  Major = %d  Minor = %d    # of AUX Bytes = %d\n",
               K5.version1,K5.version2,K5.auxbyte);
           printf("    AUX Field Data : ");
           for(n=0; n<K5.auxbyte/4; n++){
               printf(" %04X", K5.aux32[n]);
           }
           aux.size=K5.auxbyte;
           //strncpy(aux.bdat,(char *)K5.aux32,K5.auxbyte); // this is not good for in case of data 0 is included
           for(n=0; n<aux.size; n++) aux.bdat[n]=*((char *)K5.aux32+n);
           vssp_aux_parse(&aux);
           printf("\n        Format # = %d\n",aux.fmt);
           if(aux.fmt != 0)
             printf("        Filter (MHz) : %d\n",aux.filter);
             vph->filter=aux.filter;
           if(aux.fmt == 1)
             printf("        Station info : ID2(%s) name(%s)\n",aux.sid2,aux.station8);
             for(n=0; n<3; n++) vph->aux_sid2[n]=aux.sid2[n];
             for(n=0; n<9; n++) vph->aux_station[n]=aux.station8[n];
           if(aux.fmt == 1 || aux.fmt == 2 )
             printf("        PC host name :  %s\n",aux.hostname8);
             for(n=0; n<9; n++) vph->aux_hostname[n]=aux.hostname8[n];
           printf("\n");
           vph->year=K5.year;
           vph->totalday=K5.totalday;
           vph->v_major=K5.version1;
           vph->v_minor=K5.version2;
           vph->aux_size=K5.auxbyte;
           vph->aux_fmt=aux.fmt;


       }
       printf("\n");

//       printf(" FMT A/D CH  f(kHz)  TIME   seconds   # of bits\n");
//       printf("%s %2d  %2d   %4d %02d:%02d:%02d %6d  ", fmt,
//                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);

    

       i=fclose( K5.stream ) ;
 exitend2:;
       printf("\n");
       //tword+=K5.twordread;
       return ierr;
}

void print_usage(void)
{
    printf("\n");
    printf("%s (%s)\n",PROGRAM_NAME,PROGRAM_VERSION);
    printf("\n");
    printf("How to run: \n");
    printf("  data_recov [stype] K5file [sfreq adbit numch start_time [HDpat1 HDpat2]]\n");
    printf("             [options]\n");
    printf("\n");
    printf("    where\n");
    printf("       stype  ----- '32' for VSSP32, '0' or omitted case is VSSP\n");
    printf("       K5file ----- K5 data file name to edit\n");
    printf("       [following parameters are for corrupted header mode]\n");
    printf("        sfreq    ----- Sampling frequency (MHz)\n");
    printf("                       0.04,0.1,0.2,0.5,1,2,4,8,16,32,64\n");
    printf("        adbit    ----- A/D resolution (1,2,4,8)\n");
    printf("        numch    ----- # of channels (1,4)\n");
    printf("        start_time --- 1st time of data\n");
    printf("                       either 'HH:MM:SS' or 'seconds in day'\n");
    printf("        HDpat1  ------ corrupted header pattern 1 in 8 digit Hexs or '\?'\n");
    printf("                       where '\?' is a wild card for one Hex.\n");
    printf("                       (ex. 'FFFFFF\?\?') (default is 'FFFF\?\?\?\?')\n");
    printf("        HDpat2  ------ corrupted header pattern 3 in 8 digit Hexs or '\?'\n");
    printf("                       where '\?' is a wild card for one Hex.\n");
    printf("                       (ex. '8B\?\?\?\?FF')\n");
    printf("                       (default is '8B\?\?\?\?FF' for VSSP\n");
    printf("                                   '8C\?\?\?\?\?\?' for VSSP32)\n");
    printf("       options (any order)\n");
    printf("         -recov[er] --- force recover mode (non-interactive operation)\n");
    printf("         -check  ---- force check mode (non-interactive operation)\n");
    printf("         -pat1 HDpat1  --- set serach header pattern 1\n");
    printf("         -pat2 HDpat2  --- set serach header pattern 2\n");
    printf("         -vssp32 ---- set sampler to VSSP32 (default is VSSP)\n");
    printf("         -out ofile --- set output data file name after corrected\n");
    printf("         -in k5file --- set K5 file name to check or edit\n");
    printf("         -offset nbytes --- set offset bytes compulsory\n");
                                     
       //printf("           mode      ---  32: K5/VSSP32,  0: K5/VSSP (default)\n");
    printf("\n");
    printf("    created filename will be datafile+'.edit'\n");
    printf("     if option '-out ofile' is not set.'\n");
}         

void run_param_get(int argc, char* argv[], data_recov_runp_t *runp, vssp_head_t *vph)
/*
   get run parameters 
*/
{
    char cbuf[80];
    int arg_offset;

    // parameter default set 
    //default_set(runp);
    runp->mode=0;   // for VSSP
    //runp->mode=32;  // for VSSP32
    runp->pmode=0;   // 1: 1st frame data OK mode
    runp->interactive_mode=1;
    runp->compulsory_set_flag=0;
    runp->offset_compulsory_set_flag=0;
    runp->edit_mode=0;  // recover mode
    runp->check_mode=0;

    runp->fname[0]='\0';
    runp->oname[0]='\0';
    /* default corruped header pattern set for k51a */
    strcpy(runp->hdpat1,"FFFF????");  // default corrupted header pattern 1
    //strcpy(runp->hdpat2,"8B????FF");  // default corrupted header pattern 2
    strcpy(runp->hdpat2,"8B??????");  // default corrupted header pattern 2

    runp->hh=0;
    runp->mm=0;
    runp->ss=0;
    runp->fsmhz=0.0;
    runp->byteoffset=0;

    vph->sfreq=0;
    vph->adbit=0;
    vph->numch=0;
    vph->seconds=0;

    if(argc < 2) goto exithere;  // nothing to do

    /* argument analysis */
    arg_offset=0;
    if (argc>=2){
       if (strncmp(*(argv+1),"-",1)!=0){  // this is not command
          strcpy(cbuf,*(argv+1));
          //printf("%s\n",cbuf);
          if(cbuf[0]=='3' && cbuf[1]=='2'){
             runp->mode=32;
             arg_offset=1;
          }
       }
    } 
    if (argc>=2+arg_offset){
       if (strncmp(*(argv+1+arg_offset),"-",1)!=0){  // this is not command
           strcpy(runp->fname,*(argv+1+arg_offset));
       }
    }
    if (argc==2+arg_offset) runp->pmode=1;  // 1st header is OK mode

    if(runp->mode != 32){   // for VSSP
       strcpy(runp->hdpat2,"8B????FF");  // default corrupted header pattern 2
    }

    if(argc >=3+arg_offset){
       if (strncmp(*(argv+2+arg_offset),"-",1)!=0){  // this is not command
          runp->fsmhz = atof(*(argv + 2+arg_offset ));
	      vph->sfreq=(int)(runp->fsmhz*1000.0);
       }
    }
    if(argc >=4+arg_offset){
       if (strncmp(*(argv+3+arg_offset),"-",1)!=0){  // this is not command
          vph->adbit = atoi(*(argv + 3+arg_offset ));
       }
    }
    if(argc >=5+arg_offset){
       if (strncmp(*(argv+4+arg_offset),"-",1)!=0){  // this is not command
           vph->numch = atoi(*(argv + 4+arg_offset ));
       }
    }
    if(argc >=6+arg_offset){
       if (strncmp(*(argv+5+arg_offset),"-",1)!=0){  // this is not command
          strcpy(cbuf,*(argv + 5+arg_offset ));
          vph->seconds=hhmmss2sec(cbuf,&runp->hh,&runp->mm,&runp->ss);
          //printf("seconds=%d\n",vph.seconds); // debug
          //printf("hh mm ss %d %d %d\n",hh,mm,ss); // debug
          runp->compulsory_set_flag=1;
       }

    }
    if(argc >=7+arg_offset){
       if (strncmp(*(argv+6+arg_offset),"-",1)!=0){  // this is not command
          strcpy(runp->hdpat1,*(argv + 6+arg_offset ));
       }
    }
    if(argc >=8+arg_offset){
       if (strncmp(*(argv+7+arg_offset),"-",1)!=0){  // this is not command
          strcpy(runp->hdpat2,*(argv + 7+arg_offset ));
       }
    }

    /* command analysis */
    if(cmnd_get(argc,argv,"-vssp32",cbuf)==0) runp->mode=32; //VSSP32 mode
    if(cmnd_get(argc,argv,"-VSSP32",cbuf)==0) runp->mode=32; //VSSP32 mode
    if(cmnd_get(argc,argv,"-check",cbuf)==0){
        runp->check_mode=1; // check mode
        runp->interactive_mode=0;
    }
    if(cmnd_get(argc,argv,"-CHECK",cbuf)==0){
        runp->check_mode=1; // check mode
        runp->interactive_mode=0;
    }
    if(cmnd_get(argc,argv,"-edit",cbuf)==0){
        runp->edit_mode=1; // edit mode
        runp->interactive_mode=0;
    }
    if(cmnd_get(argc,argv,"-EDIT",cbuf)==0){
        runp->edit_mode=1; // edit mode
        runp->interactive_mode=0;
    }
    if(cmnd_get(argc,argv,"-recov",cbuf)==0){
        runp->edit_mode=1; // edit mode
        runp->interactive_mode=0;
    }
    if(cmnd_get(argc,argv,"-RECOV",cbuf)==0){
        runp->edit_mode=1; // edit mode
        runp->interactive_mode=0;
    }
    if(cmnd_get(argc,argv,"-out",cbuf)==0){
       strcpy(runp->oname,cbuf); //out file name 
    }
    if(cmnd_get(argc,argv,"-in",cbuf)==0){
       strcpy(runp->fname,cbuf); //input file name 
    }
    if(cmnd_get(argc,argv,"-pat1",cbuf)==0){
       strcpy(runp->hdpat1,cbuf);  
    }
    if(cmnd_get(argc,argv,"-pat2",cbuf)==0){
       strcpy(runp->hdpat2,cbuf);  
    }
    if(cmnd_get(argc,argv,"-offset",cbuf)==0){
       runp->byteoffset=atoi(cbuf);
       runp->offset_compulsory_set_flag=1;
    }

    // 

    if(runp->mode == 32){   // for VSSP
       strcpy(runp->hdpat2,"8C??????");  // default corrupted header pattern 2
    }

	// out file name creation if not set
    if(runp->oname[0]=='\0'){
	   strcpy(runp->oname,runp->fname);
	   strcat(runp->oname,".edit");
	   //printf(" Out file name: %s\n",oname);
    }
	// set 2nd sync pattern
	vph->psync2=0x8B;              // K5/VSSP
	if(runp->mode==32) vph->psync2=0x8C; // K5/VSSP32

exithere:;
    //strcpy(runp->stcode_file,dflt_stcode_file); //obsoleted on 2003-11-7
}

void monit_param(data_recov_runp_t *runp, vssp_head_t *vph)
{
    printf("fname  : %s\n",runp->fname);
    printf("oname  : %s\n",runp->oname);
    printf("mode   : %d\n",runp->mode);
    printf("pmode  : %d\n",runp->pmode);
    printf("hpdat1 : %s\n",runp->hdpat1);
    printf("hpdat2 : %s\n",runp->hdpat2);
    printf("interactive_mode : %d\n",runp->interactive_mode);
    printf("edit_mode  : %d\n",runp->edit_mode);
    printf("check_mode : %d\n",runp->check_mode);
    printf("fsmhz  : %f\n",runp->fsmhz);
    printf("hh,mm,ss : %d %d %d\n",runp->hh,runp->mm,runp->ss);
    printf(" ** vph **\n");
    printf("sfreq : %d\n",vph->sfreq);
    printf("adbit : %d\n",vph->adbit);
    printf("numch : %d\n",vph->numch);
    printf("seconds : %d\n",vph->seconds);
}

int main(int argc, char *argv[])
{
    //char fname[80];
	//char oname[100];
	char cbuf[40];
	char cw;
    //char hdpat1[20],hdpat2[20];
    //int mode, byteoffset;
    //int pmode; // processing mode
    //int arg_offset;
	//int hh,mm,ss;
	double fsmhz;
    double telapsed;           // to measure processing time
    data_recov_runp_t runp;
    vssp_head_t vph;
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    //mode=0;   // for VSSP
    //mode=32;  // for VSSP32
    //pmode=0;
    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
       print_usage();
       exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
       printf("\n");
       printf("%s (%s)\n",PROGRAM_NAME,PROGRAM_VERSION);
       printf("\n");
       exit(0);
    }


    run_param_get(argc, argv, &runp, &vph);
    //monit_param(&runp, &vph);  // for debug

    if(runp.pmode==1){  // at least 1st headder is available
        if(chkK5header(&runp, &vph) <0){
            printf(" Please run again with not automatic mode!\n");
            return 0;
        }
        fsmhz=(double)vph.sfreq/1000.0;
        sec2hhmmss(vph.seconds, &runp.hh, &runp.mm, &runp.ss);
    }

    printf("************ %s  %s ************\n",PROGRAM_NAME,PROGRAM_VERSION);
	printf("\n");
display:
	printf(" Run condition is as follows\n");
    if(runp.mode==32){
	printf("   Sampler Type             : VSSP32\n");
    } else {
	printf("   Sampler Type             : VSSP\n");
    }
	printf("   Input  K5 file           : %s\n",runp.fname);
	printf("   K5 file after edit       : %s\n",runp.oname);
	printf("   Sampling Frequency (MHz) : %f\n",runp.fsmhz);
	printf("   AD bits                  : %d\n",vph.adbit);
	printf("   # of channels            : %d\n",vph.numch);
	printf("   Start Time               : %02d:%02d:%02d (%d seconds)\n",runp.hh,runp.mm,runp.ss,vph.seconds);
    if(runp.pmode==1)
    printf("   Offset Bytes             : %d\n",runp.byteoffset);
    if(runp.pmode==0)
    printf("   Search header pattern    : %s %s\n",runp.hdpat1,runp.hdpat2);
	printf("\n");

    if(runp.interactive_mode == 1){  // interactive mode
	   printf(" Is this condition OK? ([Y]/N) ");
       cw=getchar();  // read to accept just return
       if(cw=='N' || cw=='n') {
           printf(" Enter Sampling Frequency (MHz) ----> ");
           scanf("%s",cbuf);
           runp.fsmhz=atof(cbuf);
	       vph.sfreq=(int)(runp.fsmhz*1000.0);
           printf(" Enter AD bits ----------> ");
           scanf("%d",&vph.adbit);
           printf(" Enter # of channels ----> ");
           scanf("%d",&vph.numch);
           printf(" Enter Start Time (HH:MM:SS) ----> ");
           scanf("%s",cbuf);
           vph.seconds=hhmmss2sec(cbuf,&runp.hh,&runp.mm,&runp.ss);
           printf(" Enter Offset Bytes ----> ");
           scanf("%d",&runp.byteoffset);
           cw=getchar();  // dummy read (LF code)
           goto display;
       } else {
          if(cw=='Y' || cw=='y') cw=getchar();  // dummy read (LF code)
       }
    }

    // processing start

	cstart1sec = clock();

    if(runp.pmode==0){

        if(runp.offset_compulsory_set_flag==0){
           runp.byteoffset=find_offset(runp.fname,&vph,runp.mode,runp.hdpat1,runp.hdpat2); 
	       if(runp.byteoffset < 0) return 0;
        }
       if(runp.interactive_mode == 1){ // interactive mode
	      printf("\n Continue Processing with offset %d? ([Y]/N) ",runp.byteoffset);
          cw=getchar();  // read to accept just return
          if(cw=='N' || cw=='n') return 0;
          if(cw=='Y' || cw=='y') cw=getchar();  // dummy read (LF code)
       }
       if(runp.mode == 32){
           if(chkK5header(&runp, &vph) <0){
              printf(" Please run again with not automatic mode!\n");
              return 0;
           }
       }
    }
    if(runp.interactive_mode==1){  // interactive mode
	    printf("\n");
	    printf("  1 -- Header Check Only using %d byte offset\n",runp.byteoffset);
	    printf("  2 -- Replace Header    using %d byte offset\n",runp.byteoffset);
	    printf("\n Enter your selection ([1]/2) ");
        cw=getchar();  // read to accept just return
        if(cw=='2'){
           editK5file(&runp,&vph);
	    } else {
           checkK5file(&runp,&vph);
	    }
    } else {  // non-interactive mode
	    printf("Continue Processing with offset %d \n",runp.byteoffset);
        if(runp.check_mode==1) checkK5file(&runp,&vph);
        if(runp.edit_mode==1) editK5file(&runp,&vph);
    }
    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    printf("Time elapsed for processing is %f sec\n",telapsed);
    return 0;
}
